/**
 * 
 */
package binpackage;

import exceptions.AbortException;
import exceptions.GreedyException;
import exceptions.NoContextException;

/**
 * Strategy design pattern: This abstract class describes an anonymous greedy solver.
 * @author dennis
 */
public abstract class AbstractGreedy extends ProblemWorker{

	/**
	 * The constructor initializes the generator instance with all data it needs for operation.
	 * All data (including the list of bins and the list of elements) is taken from the given context.
	 * 
	 * @param p_cxt Context object with all necessary data
	 */
	public AbstractGreedy(Context p_cxt) {
		super(p_cxt);
	}
	
	/**
	 * This method performs exactly one greedy solving step. Then it stores its state to the
	 * greedy object so that it can continue with the next call. It is safe to assume
	 * that the next call to greedyStep will refer to the same problem. If a new problem
	 * was generated a new greedy object would be created.
	 * 
	 * @throws GreedyException A sub-class of this gets thrown if something goes wrong.
	 * @throws AbortException This exception gets thrown when the solving reaches its end.
	 *         This means a complete solution has been found.
	 * @throws NoContextException A valid context needs to be provided prior to calling
	 *                            this method.
	 */
	public abstract void greedyStep() throws GreedyException, AbortException, NoContextException;
	
}
