/**
 * 
 */
package binpackage;

/**
 * This class represents an element. Internally it's represented by a double number.
 * @author dennis
 */
public class Element implements Comparable<Element>{

	private double dValue = 0.0;
	private int iBinNr = -1;
	
	/**
	 * Constructor. An Element always need to have a value. Using this container
	 * the Element won't have a bin container.
	 * @param p_dValue The value of the Element
	 */
	public Element(double p_dValue) {
		this.dValue = p_dValue;
	}

	/**
	 * Constructor. An Element always need to have a value. They can be part of a bin.
	 * @param p_dValue The value of the Element
	 * @param p_iBinNr The number of the bin
	 */
	public Element(double p_dValue, int p_iBinNr) {
		this.dValue = p_dValue;
		this.iBinNr = p_iBinNr;
	}

	/**
	 * Copy constructor. Creates a new Element as a copy of the given Element.
	 * @param p_eTemplate A template Element
	 */
	public Element(Element p_eTemplate) {
		this(p_eTemplate.getValue(), p_eTemplate.getBinNr());
	}
	
	/**
	 * @return the Value
	 */
	public double getValue() {
		return this.dValue;
	}

	/**
	 * @param p_dValue The new value of the element
	 */
	public void setValue(double p_dValue) {
		this.dValue = p_dValue;
	}

	/**
	 * @return the bin number of this element. If the element belongs to no bin a value
	 * less than zero gets returned.
	 */
	public int getBinNr() {
		return this.iBinNr;
	}

	/**
	 * @param p_iBinrNr The number of the bin of this element. Set it to a vlue less
	 * than zero if it belongs to no element.
	 */
	public void setBinNr(int p_iBinNr) {
		this.iBinNr = p_iBinNr;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		long temp = Double.doubleToLongBits(dValue);;
		
		result = prime * result + (int) (temp ^ (temp >>> 32));
		result = prime * result + iBinNr;
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		
		final Element other = (Element) obj;
		
		if (Double.doubleToLongBits(dValue) != Double.doubleToLongBits(other.dValue))
			return false;
		
		if (iBinNr != other.iBinNr)
			return false;
		
		return true;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "[" + this.iBinNr + "]: " + this.dValue;
	}

	/**
	 * This method compares two Elements by their value no matter of their parent bin. 
	 */
	public int compareTo(Element e) {
		int i_return = 0;
		
		if(this.dValue < e.getValue()) {
			i_return =  -1;
		} else if(this.dValue ==  e.getValue()) {
			i_return =  0;
		} else if(this.dValue > e.getValue()) {
				i_return = 1;
		}
		
		return i_return;
	}
}
