/**
 * 
 */
package binpackage;

import java.util.LinkedList;
import java.util.Random;
import exceptions.NoContextException;
import exceptions.ProblemGeneratorException;
import exceptions.ProblemGeneratorLessElementsThanBinsException;

/**
 * This is a simple problem generator. It starts with assuming that each bin is
 * completely filled. Then it removes enough random values from the bins until the
 * required amount of elements has been found. It is assured that all bins can
 * be filled up to 100% again.
 * @author dennis
 */
public final class FastProblemGenerator extends AbstractProblemGenerator {

	public static final String strName = "Fast";
	
	private int i_nBins;                      // Amount of bins
	private int i_nElements;                  // Amount of elements
	
	private Random rnd;                       // Random number generator
	private LinkedList<Element> llElements;   // List of all elements
	
	
	/* (non-Javadoc)
	 * @see binpackage.AbstractProblemGenerator#AbstractProblemGenerator(binpackage.Context)
	 */
	public FastProblemGenerator(Context p_cxt) {
		super(p_cxt);
		
		this.i_nBins = p_cxt.getAmountBins();
		this.i_nElements = p_cxt.getAmountElements();
		
		this.llElements = p_cxt.getElements();
		this.rnd = new Random();
	}
	
	
	/* (non-Javadoc)
	 * @see binpackage.AbstractProblemGenerator#generateProblem()
	 */
	@Override
	public void generateProblem() throws NoContextException,
	                                     ProblemGeneratorException {
		int iBin = -1;
		double newElementValue;
		double[] dBinCapacity = new double[this.i_nBins];
		
		// Check faulty start conditions
		this.checkStartConditions();
		
		if (i_nElements < i_nBins) {
			throw new ProblemGeneratorLessElementsThanBinsException();
		}
		
		// Assume that each bin starts with full remaining capacity
		for (int i = 0; i < dBinCapacity.length; i++) {
			dBinCapacity[i] = this.cxt.getBinCapacity();
		}
		
		/* Progressively remove units from each bins until required amount of
		 * Elements has been reached. It is assured that always less than 75%
		 * of the remaining capacity of a bin are removed from it.
		 */
		for (int i = 0; i < this.i_nElements; i++) {	
			iBin++;
			iBin = iBin % this.i_nBins;
			
			do {
				newElementValue = (this.rnd.nextDouble() * 0.75) * dBinCapacity[iBin];
			} while (newElementValue == 0.0);
			
			dBinCapacity[iBin] -= newElementValue;
			this.llElements.add(new Element(newElementValue));
		}
	}

}
