 /**
 * 
 */
package binpackage;

import java.util.Random;
import exceptions.AbortException;
import exceptions.NoContextException;
import exceptions.OptimizerException;
import java.util.Collections;
import java.util.LinkedList;

/**
 * This is an evolutionary optimizer. It uses evolutionary techniques to find an optimal
 * solution to the given problem.
 * @author mfab
 */
public final class LocalOptimizer extends AbstractOptimizer {
	
	public static final String strName = "Local";

	private LinkedList<Element> ll_elements;     // Elements to be sorted
	private Random rnd;                          // Randomizer
	
	private boolean b_sorted = false;           // Elements already sorted
	
	// TODO probably check that there are more than two bins before attempting to optimize
	private int i_nElements;                    // Amount of elements
	private int i_swapc1;                   	// 1st swap candidate
	private int i_swapc2;                   	// 2nd swap candidate
	private Element e_swapc1;
	private Element e_swapc2;
	private Context c_newsolution;				// Context to compare the fitness function of generated solution
	private LinkedList<Element> ll_newsolution; // list of new elements for new context
	//private int i_nSwaps;						// how many swaps to perform read from amount mutation per child
	
	/* (non-Javadoc)
	 * @see binpackage.AbstractOptimizer#AbstractOptimizer(binpackage.Context)
	 */
	public LocalOptimizer(Context p_cxt) {
		super(p_cxt);
		
		this.i_nElements = p_cxt.getAmountElements();
		this.ll_elements = p_cxt.getElements();
		this.rnd = new Random();
		//this.i_nSwaps = 1 + p_cxt.getAmountMutationPerChild();
	}
	
	// TODO Replace OptimizerExcption with specific sub-classes of it.
	//      Don't throw an optimizerException itself. Always throw a sub-class.
	@Override
	public void optimizerStep() throws OptimizerException,
	                                   AbortException,
	                                   NoContextException {
		if (this.cxt == null) {
			throw new NoContextException();
		}
		// TODO amount of dead children (implement correctly or leave it out)
		// TODO
		// Check abort condition here, then
		// TODO find correct abort condition (no changes to the last run... etc.)
		if (cxt.getFitness() == 0)
		throw new AbortException();
		// if a final solution has been found.
	
		// check if all elements are sorted - otherwise sort them
		if (b_sorted == false) {
			Collections.sort(ll_elements);
			b_sorted = true;
		}
		

		// swap all elements systematically
		
		//while (i_nSwaps > 0) {

			// loop all elements to swap
			for (Element e1 : ll_elements) {

				//i_nSwaps--;
				//if (i_nSwaps <= 0) break;  // else do the swap

				// find first swap-candidate
				i_swapc1 = rnd.nextInt(i_nElements - 1);
				e_swapc1 = e1;

				// find 2nd swap-candidate with Hamming-distance 2
				// must not be equal to the 1st candidate, not leave the bounds of the elements-list
				// and must be in a different bin
				// added abort integer value to prevent infinite loop when all elements with
				// hamming distance 2 are in the same bin
				// TODO alle elemente mit Hamming 2 durchtesten und swappen, wenn besser!!!
				int i_abort = 0;
				do {
					do {
						i_swapc2 = (rnd.nextInt(4) - 2) + i_swapc1;
					}  // inner while necessary to prevent ArrayIndexOutOfBoundsException
					while (i_swapc2 < 0
							|| i_swapc2 == i_swapc1
							|| i_swapc2 > (i_nElements-1));
					e_swapc2 = ll_elements.get(i_swapc2);
					if (i_abort++ > 8) break;
				}
				while (i_swapc2 < 0
						|| i_swapc2 == i_swapc1
						|| i_swapc2 > (i_nElements-1)
						|| e_swapc1.getBinNr() == e_swapc2.getBinNr());

				// generate new Context to compare fitness of old and new solution
				// used for "rollback" if changes to current context are worse than before

				try {
					c_newsolution = (Context) cxt.clone();
				} catch (CloneNotSupportedException cnse) { System.out.println("cnse: " + cnse); };

				// swap bins in current/given/old context
				ll_newsolution = cxt.getElements();

				// swap bins of elements
				int i_swaphelp = ll_newsolution.get(i_swapc1).getBinNr(); // get bin number of first element
				//System.out.println("swapc1-bin: " + ll_newsolution.get(i_swapc1).getBinNr());
				//System.out.println("swapc2-bin: " + ll_newsolution.get(i_swapc2).getBinNr());
				ll_newsolution.get(i_swapc1).setBinNr(ll_newsolution.get(i_swapc2).getBinNr()); // set bin of first element to bin of second element
				ll_newsolution.get(i_swapc2).setBinNr(i_swaphelp);  // set bin of second element to saved bin of first element

				// Debug
				//System.out.println("before: " + cxt.getFitness());
				//System.out.println("after swap: " + c_newsolution.getFitness());

				// check if current context is a better solution than the newly swapped one
				// if current is better or equal do nothing and raise amount of dead children
				if (cxt.getFitness() <= c_newsolution.getFitness()) {
					//cxt.setAmountDeadChildren(cxt.getAmountDeadChildren()+1);
					//System.out.print("" + (cxt.getAmountDeadChildren()+1) + ", ");
				}
				// otherwise take the newly swapped solution as current one
				else {cxt.setElements(c_newsolution.getElements());};
			} // end loop through all elements	
		//} // while
		
	} // optimizerStep()
	
	// TODO in or out?!
	//public int getAmountDeadChildren() {
	//	return cxt.getAmountDeadChildren();
	//}
	
}
