/**
 * 
 */
package binpackage;

import java.net.URL;
import java.text.ParseException;
import java.awt.HeadlessException;
import java.awt.Container;
import java.awt.BorderLayout;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.awt.Graphics;
import java.awt.Color;
import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JSpinner;
import javax.swing.JToolBar;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.border.EmptyBorder;
import java.util.Collections;
import java.util.Vector;
import java.util.List;
import java.text.NumberFormat;
import com.jeta.forms.components.panel.FormPanel;
import com.jgoodies.looks.Options;
import com.jgoodies.looks.HeaderStyle;


/**
 * This is the main window. It's controlled by a MainFrameController instance.
 * @author dennis
 */
public class MainFrame extends JFrame {
	
	protected static final long serialVersionUID = 1;
	protected static final String strFrameFile = Globals.strResDir + "MainFrame.jfrm";
	protected static final String strIconName = "truck-icon.gif";
	
	// ID-Strings to access form elements
	protected static final String ID_ProblemCreate = "btnProblemCreate";
	protected static final String ID_GreedySingleStep = "btnGreedySingleStep";
	protected static final String ID_GreedyStop = "btnGreedyStop";
	protected static final String ID_GreedyFinish = "btnGreedyFinish";
	protected static final String ID_OptimizerSteps = "spinOptimizerSteps";
	protected static final String ID_OptimizerSingleStep = "btnOptimizerSingleStep";
	protected static final String ID_OptimizerStop = "btnOptimizerStop";
	protected static final String ID_OptimizerHold = "btnOptimizerHold";
	
	protected static final String ID_ProblemStrategy = "cbxProblemStrategy";
	protected static final String ID_ProblemAmountBins = "spinProblemAmountBins";
	protected static final String ID_ProblemAmountElements = "spinProblemAmountElements";
	protected static final String ID_GreedyStrategy = "cbxGreedyStrategy";
	protected static final String ID_OptimizerStrategy = "cbxOptimizerStrategy";
	protected static final String ID_OptimizerPopulationSize = "spinOptimizerPopulationSize";
	protected static final String ID_OptimizerOffspringSize = "spinOptimizerOffspringSize";
	protected static final String ID_OptimizerMutationCount = "spinOptimizerMutationCount";
	
	protected static final String ID_PopulationBrowser = "cbxPopulationBrowser";
	protected static final String ID_DisplaySelectedChild = "btnDisplaySelectedChild";
	
	protected static final String ID_VisualizationLabel = "lblVisualization";
	protected static final String ID_VisualizationFitnessValue = "lblFitness";
	protected static final String ID_VisualizationElementList = "lstUnboundElements";
	
	protected static final String ID_LastDuration = "txtLastDuration";
	protected static final String ID_TotalDuration = "txtTotalDuration";
	protected static final String ID_AmountDead = "txtAmountDead";
	protected static final String ID_MinMissing = "txtMinMissing";
	protected static final String ID_MaxMissing = "txtMaxMissing";
	protected static final String ID_PopulationGraph = "lblPopulationGraph";
	
	// Form elements
	protected Container cp;
	protected JToolBar tbDisplay;
	protected FormPanel fpMain;
	
	protected JButton btnZoomIn;
	protected JButton btnZoomOut;
	protected JButton btnZoomNormal;
	protected JButton btnZoomFit;
	
	protected JButton btnProblemCreate;
	
	protected JButton btnGreedySingleStep;
	protected JButton btnGreedyStop;
	protected JButton btnGreedyFinish;
	
	protected JSpinner spinOptimizerSteps;
	protected JButton btnOptimizerSingleStep;
	protected JButton btnOptimizerStop;
	protected JButton btnOptimizerHold;
	
	protected JComboBox cbxProblemStrategy;
	protected JSpinner spinProblemAmountBins;
	protected JSpinner spinProblemAmountElements;
	
	protected JComboBox cbxGreedyStrategy;
	
	protected JComboBox cbxOptimizerStrategy;
	protected JSpinner spinOptimizerPopulationSize;
	protected JSpinner spinOptimizerOffspringSize;
	protected JSpinner spinOptimizerMutationCount;
	
	protected JComboBox cbxPopulationBrowser;
	protected JButton btnDisplaySelectedChild;
	
	protected JLabel lblVisualization;
	protected JList lstVisualizationElements;
	protected JLabel lblVisualizationFitness;
	
	protected JLabel txtLastDuration;
	protected JLabel txtTotalDuration;
	protected JLabel txtAmountDead;
	protected JLabel txtMinMissing;
	protected JLabel txtMaxMissing;
	protected JLabel lblPopulationGraph;
	
	// Other attributes
	protected MainFrameController mfcController;
	protected ProblemWorkerController pwcController;
	protected ContextRenderer crRenderer;
	protected Context cxtLastDrawn = null;
	protected Vector<Context> vCurrentPopulation = new Vector<Context>();
	
	/**
	 * Constructor.
	 * @throws HeadlessException
	 */
	public MainFrame() throws HeadlessException {
		// Initializing the JFrame
		super();
		
		this.setTitle(Globals.strAppTitle);
		
		String strImage = Globals.strImgDir + strIconName;
		URL urlImage = MainFrame.class.getResource(strImage);
		
		if (urlImage != null) {
			ImageIcon icon = new ImageIcon(urlImage);
			this.setIconImage(icon.getImage());
		} else {
			System.err.println("Image not found: " + MainFrame.strIconName);
		}
		
		// The window consists of several ToolBars and the main panel
		this.cp = this.getContentPane();
		this.cp.setLayout(new BorderLayout());
		
		// Add the ToolBar (HeaderStlye.BOTH for menu bar and tool bar !!)
		this.tbDisplay = new JToolBar("Display settings");
		this.tbDisplay.putClientProperty(Options.HEADER_STYLE_KEY, HeaderStyle.SINGLE);
		
		this.btnZoomIn = makeToolBarButton("gtk-zoom-in.png", "Zoom In", "Magnify the visualization");
		this.btnZoomOut = makeToolBarButton("gtk-zoom-out.png", "Zoom Out", "Shrink the visualization");
		this.btnZoomNormal = makeToolBarButton("gtk-zoom-100.png", "Zoom Normal", "Reset the visualization size to its default");
		this.btnZoomFit = makeToolBarButton("gtk-zoom-fit.png", "Zoom Fit", "Make the visualization fit the screen");
		
		this.tbDisplay.add(btnZoomIn);
		this.tbDisplay.add(btnZoomOut);
		this.tbDisplay.add(btnZoomNormal);
		this.tbDisplay.add(btnZoomFit);
		
		this.cp.add(this.tbDisplay, BorderLayout.PAGE_START);
		
		// Include the main panel
		this.fpMain = new FormPanel(MainFrame.strFrameFile);
		this.cp.add(this.fpMain, BorderLayout.CENTER);
		
		// Get references to the most common elements of the main panel
		this.getMainPanelReferences();
		
		// Fill ComboBoxes
		this.pwcController = new ProblemWorkerController();
		this.fillStrategyLists();
		
		// Initialize population display
		this.refreshPopulationBrowser();
		this.updatePopulationGraph();
				
		// Create a new controller to respond to UI events
		this.updatePanelButtons();
		this.crRenderer = new ContextRenderer();
		this.mfcController = new MainFrameController(this, this.pwcController);
	}

	/**
	 * This method creates a new JButton with a text and an image. It's meant to create
	 * buttons for the ToolBar.
	 * @param p_strImage File name of the button image relative to the resources directory.
	 * @param p_strText Text of the button.
	 * @param p_strToolTip ToolTip for the button.
	 * @return A new JButton to be inserted in a JToolBar.
	 */
	protected JButton makeToolBarButton(String p_strImage, String p_strText, String p_strToolTip) {
		JButton btn = new JButton();
		btn.setBorder(new EmptyBorder(4, 4, 4, 4));
		btn.setToolTipText(p_strToolTip);
		
		String strImage = Globals.strImgDir + p_strImage;
		URL urlImage = MainFrame.class.getResource(strImage);
		
		if (urlImage != null) {
			btn.setIcon(new ImageIcon(urlImage, p_strText));
			btn.setText(p_strText);
		} else if (p_strImage.equals("")) {
			btn.setText(p_strText);
		} else {
			btn.setText(p_strText);
			System.err.println("Image not found: " + p_strImage);
		}
		
		return btn;
	}
	
	protected void getMainPanelReferences() {
		this.btnProblemCreate = (JButton) this.fpMain.getButton(ID_ProblemCreate);
		
		this.btnGreedySingleStep = (JButton) this.fpMain.getButton(ID_GreedySingleStep);
		this.btnGreedyStop = (JButton) this.fpMain.getButton(ID_GreedyStop);
		this.btnGreedyFinish = (JButton) this.fpMain.getButton(ID_GreedyFinish);
		
		this.spinOptimizerSteps = (JSpinner) this.fpMain.getSpinner(ID_OptimizerSteps);
		this.btnOptimizerSingleStep = (JButton) this.fpMain.getButton(ID_OptimizerSingleStep);
		this.btnOptimizerStop = (JButton) this.fpMain.getButton(ID_OptimizerStop);
		this.btnOptimizerHold = (JButton) this.fpMain.getButton(ID_OptimizerHold);
		
		this.cbxProblemStrategy = (JComboBox) this.fpMain.getComboBox(ID_ProblemStrategy);
		this.spinProblemAmountBins = (JSpinner) this.fpMain.getSpinner(ID_ProblemAmountBins);
		this.spinProblemAmountElements = (JSpinner) this.fpMain.getSpinner(ID_ProblemAmountElements);
		
		this.cbxGreedyStrategy = (JComboBox) this.fpMain.getComboBox(ID_GreedyStrategy);
		
		this.cbxOptimizerStrategy = (JComboBox) this.fpMain.getComboBox(ID_OptimizerStrategy);
		this.spinOptimizerPopulationSize = (JSpinner) this.fpMain.getSpinner(ID_OptimizerPopulationSize);
		this.spinOptimizerOffspringSize = (JSpinner) this.fpMain.getSpinner(ID_OptimizerOffspringSize);
		this.spinOptimizerMutationCount = (JSpinner) this.fpMain.getSpinner(ID_OptimizerMutationCount);
		
		this.cbxPopulationBrowser = (JComboBox) this.fpMain.getComboBox(ID_PopulationBrowser);
		this.btnDisplaySelectedChild = (JButton) this.fpMain.getButton(ID_DisplaySelectedChild);
			
		this.lblVisualization = (JLabel) this.fpMain.getComponentByName(ID_VisualizationLabel);
		this.lstVisualizationElements = (JList) this.fpMain.getList(ID_VisualizationElementList);
		this.lblVisualizationFitness = (JLabel) this.fpMain.getLabel(ID_VisualizationFitnessValue);
		
		this.txtLastDuration = (JLabel) this.fpMain.getLabel(ID_LastDuration);
		this.txtTotalDuration = (JLabel) this.fpMain.getLabel(ID_TotalDuration);
		this.txtAmountDead = (JLabel) this.fpMain.getLabel(ID_AmountDead);
		this.txtMinMissing = (JLabel) this.fpMain.getLabel(ID_MinMissing);
		this.txtMaxMissing = (JLabel) this.fpMain.getLabel(ID_MaxMissing);
		this.lblPopulationGraph = (JLabel) this.fpMain.getLabel(ID_PopulationGraph);		
	}
	
	/**
	 * This method uses a ProblemWorkerController to search for all problem generation /
	 * greedy / optimizing strategies. It fills the appropriate ComboBoxes on the MainFrame.
	 */
	protected void fillStrategyLists() {
		// Clear all DropDown lists
		this.cbxProblemStrategy.removeAllItems();
		this.cbxGreedyStrategy.removeAllItems();
		this.cbxOptimizerStrategy.removeAllItems();
		
		// Fill problem generator list
		for (ProblemWorkerDescription pwd : this.pwcController.getProblemGeneratorDescriptions()) {
			this.cbxProblemStrategy.addItem(pwd.getName());
		}
		
		// Fill greedy solver list
		for (ProblemWorkerDescription pwd : this.pwcController.getGreedyDescriptions()) {
			this.cbxGreedyStrategy.addItem(pwd.getName());
		}
		
		// Fill optimizer list
		for (ProblemWorkerDescription pwd : this.pwcController.getOptimizerDescriptions()) {
			this.cbxOptimizerStrategy.addItem(pwd.getName());
		}
	}
	
	/**
	 * This method enables or disables some of the panel buttons according to the current
	 * state of the problem generator, the greedy solution finder and the optimizer. 
	 */
	protected void updatePanelButtons() {
		// Declare one enabled / disabled flag per button
		boolean bProblemCreate = false;
		
		boolean bGreedySingleStep = false;
		boolean bGreedyStop = false;
		boolean bGreedyFinish = false;
		
		boolean bOptimizerSingleStep = false;
		boolean bOptimizerStop = false;
		boolean bOptimizerHold = false;
		
		boolean bGreedyParameters = false;
		boolean bOptimizerParameters = false;		
		
		// State of each ProblemWorker
		ProblemWorkerState pwsProblemGenerator = this.pwcController.getProblemGeneratorState();
		ProblemWorkerState pwsGreedy = this.pwcController.getGreedyState();
		ProblemWorkerState pwsOptimizer = this.pwcController.getOptimizerState();
		
		// Set problem generator flags according to ProblemWorkerState
		bProblemCreate = true;
		
		// Set greedy flags according to ProblemWorkerState
		if (pwsProblemGenerator == ProblemWorkerState.FINISHED) {
			if (pwsGreedy == ProblemWorkerState.STOPPED) {
				bGreedyParameters = true;
				bGreedySingleStep = true;
				bGreedyStop = false;
				bGreedyFinish = true;
				
			} else if (pwsGreedy == ProblemWorkerState.RUNNING) {
				bGreedyParameters = false;
				bGreedySingleStep = true;
				bGreedyStop = true;
				bGreedyFinish = true;
				
			} else if (pwsGreedy == ProblemWorkerState.FINISHED) {
				bGreedyParameters = false;
				bGreedySingleStep = false;
				bGreedyStop = false;
				bGreedyFinish = false;
				
			}
		} else {
			bGreedyParameters = false;
			bGreedySingleStep = false;
			bGreedyStop = false;
			bGreedyFinish = false;
		}
		
		// Set optimizer flags according to ProblemWorkerState
		if ((pwsProblemGenerator == ProblemWorkerState.FINISHED) && (pwsGreedy == ProblemWorkerState.FINISHED)) {
			if (pwsOptimizer == ProblemWorkerState.STOPPED) {
				bOptimizerParameters = true;
				bOptimizerSingleStep = true;
				bOptimizerStop = false;
				bOptimizerHold = false;
				
			} else if (pwsOptimizer == ProblemWorkerState.RUNNING) {
				bOptimizerParameters = false;
				bOptimizerSingleStep = true;
				bOptimizerStop = true;
				bOptimizerHold = true;
				
			} else if (pwsOptimizer == ProblemWorkerState.FINISHED) {
				bOptimizerParameters = false;
				bOptimizerSingleStep = false;
				bOptimizerStop = false;
				bOptimizerHold = true;
				
			}
		} else {
			bOptimizerParameters = false;
			bOptimizerSingleStep = false;
			bOptimizerStop = false;
			bOptimizerHold = false;
		}
		
		// Apply flags to buttons
		this.btnProblemCreate.setEnabled(bProblemCreate);
		
		this.cbxGreedyStrategy.setEnabled(bGreedyParameters);
		this.btnGreedySingleStep.setEnabled(bGreedySingleStep);
		this.btnGreedyStop.setEnabled(bGreedyStop);
		this.btnGreedyFinish.setEnabled(bGreedyFinish);
		
		this.cbxOptimizerStrategy.setEnabled(bOptimizerParameters);
		this.spinOptimizerMutationCount.setEnabled(bOptimizerParameters);
		this.spinOptimizerOffspringSize.setEnabled(bOptimizerParameters);
		this.spinOptimizerPopulationSize.setEnabled(bOptimizerParameters);
		
		this.spinOptimizerSteps.setEnabled(bOptimizerSingleStep);
		this.btnOptimizerSingleStep.setEnabled(bOptimizerSingleStep);
		this.btnOptimizerStop.setEnabled(bOptimizerStop);
		this.btnOptimizerHold.setEnabled(bOptimizerHold);
	}
	
	/**
	 * This method checks the input of all input fields and resets them if an invalid
	 * input was detected.
	 */
	protected void checkInputFields() {
		this.assureSpinnerGreaterZero(this.spinProblemAmountBins);
		this.assureSpinnerGreaterZero(this.spinProblemAmountElements);
		this.assureSpinnerGreaterZero(this.spinOptimizerSteps);
		this.assureSpinnerGreaterZero(this.spinOptimizerPopulationSize);
		this.assureSpinnerGreaterZero(this.spinOptimizerOffspringSize);
		this.assureSpinnerGreaterZero(this.spinOptimizerMutationCount);
	}
	
	/**
	 * This method checks that the value of the given JSpinner is valid and greater
	 * than zero.
	 * @param p_spSpinner
	 */
	private void assureSpinnerGreaterZero(JSpinner p_spSpinner) {
		try {
			p_spSpinner.commitEdit();
		} catch (ParseException e) {
			JComponent jcEditor = p_spSpinner.getEditor();
			
			if (jcEditor instanceof JSpinner.DefaultEditor) {
				((JSpinner.DefaultEditor) jcEditor).getTextField().setValue(this.spinOptimizerOffspringSize.getValue());
			}
		}
		
		if ((Integer) p_spSpinner.getValue() < 0) {
			p_spSpinner.setValue(0);
		}
	}
	
	/**
	 * This method queries the current optimizer for its population a refreshes
	 * the drop down list in which population members can be selected for display.
	 */
	protected void refreshPopulationBrowser() {
		ProblemWorkerController pwc = null;
		ProblemWorkerState pwsOptimizer = null;
		AbstractOptimizer optimizer = null;
		List<Context> lPopulation = new Vector<Context>();
		
		int iSelectedIx = 0;
		int iCurrent = 0;
		int iAmountChildren = 0;

		// Make sure to start with an empty list
		// this.cbxPopulationBrowser.removeAll() causes drawing errors (!!!)
		int iCount = this.cbxPopulationBrowser.getItemCount();
		for (int i = 0; i < iCount; i++) {
			this.cbxPopulationBrowser.removeItemAt(0);
		}
		
		this.vCurrentPopulation.clear();
		
		if (this.mfcController != null) {
			// Get ProblemWorkerController
			pwc = this.mfcController.getProblemWorkerController();
			
			if (pwc != null) {
				// Get optimizer state
				pwsOptimizer = pwc.getOptimizerState();
		
				// Get current optimizer
				optimizer = pwc.getOptimizer();
		
				if (optimizer != null) {
					// Ask for the optimizers population members
					lPopulation = optimizer.getPopulation();
					iAmountChildren = lPopulation.size();
				}
			}
		}
		
		// Disable population browser if there's nothing to show
		if ((pwc == null) || (optimizer == null) || (pwsOptimizer == ProblemWorkerState.STOPPED) || (pwsOptimizer == null) || (iAmountChildren < 1)) {
			this.btnDisplaySelectedChild.setEnabled(false);
			this.cbxPopulationBrowser.setEnabled(false);
			return;
		}
		
		// Otherwise update selection list
		this.btnDisplaySelectedChild.setEnabled(true);
		this.cbxPopulationBrowser.setEnabled(true);
		
		iCurrent = 0;
		
		for (Context cxt : lPopulation) {
			double dFitness = cxt.getFitness();
			this.cbxPopulationBrowser.addItem("" + dFitness);
			this.vCurrentPopulation.add(cxt);
			
			if (cxt == this.cxtLastDrawn) {
				iSelectedIx = iCurrent;
			}
			
			iCurrent++;
		}
		
		// Select the currently displayed Context
		this.cbxPopulationBrowser.setSelectedIndex(iSelectedIx);
	}
	
	/**
	 * This method finds a Context to render. It first looks if there has already
	 * been a Context rendered. If not it finds the Context of the current problem.
	 * @returns a drawable Context.
	 */
	private Context findDrawableContext() {
		Context cxt;
		
		if (this.cxtLastDrawn != null) {
			cxt = this.cxtLastDrawn;
		} else {
			cxt = this.mfcController.getContext();
		}
		
		return cxt;
	}
	/**
	 * This method makes use of a ContextRenderer in order to update the displayed
	 * visualization. 
	 */
	protected void updateVisualization() {
		Context cxt = this.findDrawableContext();
		
		if (cxt == null) {
			return;
		}
		
		this.updateVisualization(cxt);
	}
	
	/**
	 * This method only updates the drawing of the currently displayed problem.
	 */
	protected void updateDrawing() {
		Context cxt = this.findDrawableContext();
		
		if (cxt == null) {
			return;
		}
		
		this.updateDrawing(cxt);
	}
	
	/**
	 * This method only redraws the currently displayed statistics.
	 */
	protected void updateStatistics() {
		Context cxt = this.findDrawableContext();
		
		if (cxt == null) {
			return;
		}
		
		this.updateStatistics(cxt);
		
	}
	
	/**
	 * This method is called in order to visualize another Context than the current
	 * one. It's meant to provide a simple means to implement a population browser.
	 * @param p_cxt The Context to display.
	 */
	protected void updateVisualization(Context p_cxt) {
		this.cxtLastDrawn = p_cxt;
		
		// Display fitness value
		this.lblVisualizationFitness.setText("" + p_cxt.getFitness());
		
		// Display unpacked Elements
		Vector<Double> vElements = new Vector<Double>();
		
		for (Element e : p_cxt.getElements()) {
			if (e.getBinNr() < 0) {
				vElements.add(e.getValue());
			}
		}
		
		Collections.sort(vElements);
		Collections.reverse(vElements);
		
		this.lstVisualizationElements.removeAll();
		this.lstVisualizationElements.setListData(vElements);
		
		// Visualize packed Elements
		this.updateDrawing(p_cxt);
		
		// Update statistics
		this.updateStatistics(p_cxt);
		
		// Update population graph
		this.updatePopulationGraph();
		
		// Update population browser
		this.refreshPopulationBrowser();
	}
	
	/**
	 * As opposed to updateVisualization() this method only updates the drawing
	 * of the current problem. No update is performed for the fitness value or
	 * the list of all unassigned Elements.
	 */
	protected void updateDrawing(Context p_cxt) {
		this.cxtLastDrawn = p_cxt;
		this.crRenderer.setContext(p_cxt);
		
		Image img = this.crRenderer.render();
		ImageIcon icon = new ImageIcon(img);
		
		lblVisualization.setIcon(icon);
	}
	
	/**
	 * This method queries the current Context for some statistics and presents
	 * it on the main frame.
	 */
	protected void updateStatistics(Context p_cxt) {	
		this.cxtLastDrawn = p_cxt;
		
		long lLastDuration = p_cxt.getLastDurationMilliSeconds();
		long lTotalDuration = p_cxt.getTotalDurationMilliSeconds();
		int iAmountDead = p_cxt.getAmountDeadChildren();
		double dMinMissing = p_cxt.getMinMissing();
		double dMaxMissing = p_cxt.getMaxMissing();
		
		NumberFormat nf = NumberFormat.getInstance();
		nf.setMaximumFractionDigits(5);
		
		this.txtLastDuration.setText(nf.format(lLastDuration));
		this.txtTotalDuration.setText(nf.format(lTotalDuration));
		this.txtAmountDead.setText(nf.format(iAmountDead));
		this.txtMinMissing.setText(nf.format(dMinMissing));
		this.txtMaxMissing.setText(nf.format(dMaxMissing));
	}
	
	/**
	 * This method queries the current optimizer for a list of all fitness values
	 * of its population. The list is visualized in a graph.
	 */
	protected void updatePopulationGraph() {
		List<Context> lPopulation = new Vector<Context>();
		
		// Create new image
		int iFrameWidth = 10;
		int iWidth = 190;
		int iHeight = 90;
		
		Image img = new BufferedImage(iWidth, iHeight, BufferedImage.TYPE_INT_RGB);
		Graphics graphics = img.getGraphics();
		
		graphics.setColor(Color.WHITE);
		graphics.fillRect(0, 0, iWidth, iHeight);

		// Check whether drawing is possible at all
		boolean bDraw = true;
		
		MainFrameController mfc = this.mfcController;
		if (mfc == null) {
			bDraw = false;
			
		} else {
			ProblemWorkerController pwc = mfc.getProblemWorkerController();
			if (pwc == null) {
				bDraw = false;
				
			} else {
				ProblemWorkerState pws = pwc.getOptimizerState();
				AbstractOptimizer optimizer = pwc.getOptimizer();
				
				if ((optimizer == null) || (pws == null) || (pws == ProblemWorkerState.STOPPED)){
					bDraw = false;
					
				} else {
					optimizer.updatePopulationStatistics();
					lPopulation = optimizer.getPopulation();
					int iAmountChildren = lPopulation.size();
					
					if (iAmountChildren < 1) {
						bDraw = false;
					}
				}
			}
		}
		
		if (bDraw == true) {
			// Find bounding fitness
			double dMinFitness = Double.MAX_VALUE;
			double dMaxFitness = 0;
			double dSpan = 0;
			
			for (Context c : lPopulation) {
				double dFitness = c.getFitness();
				
				if (dFitness < dMinFitness) {
					dMinFitness = dFitness;
				}
				
				if (dFitness > dMaxFitness) {
					dMaxFitness = dFitness;
				}
			}
			
			dSpan = Math.abs((dMaxFitness - dMinFitness));
					
			// Draw graph
			graphics.setColor(Color.BLACK);
			
			boolean bFirst = true;
			double dXStep = (iWidth / lPopulation.size());
			double dX = -dXStep;
			int iPrevX = (int) -dXStep;
			int iPrevY = 0;
			int iX = (int) -dXStep;
			int iY = 0;
			
			for (Context c : lPopulation) {
				double dFitness = c.getFitness();
				
				dX += dXStep;
				iX = (int) dX;
				iY = (int) (iHeight - (((dFitness - dMinFitness) / dSpan) * iHeight));
				
				if (bFirst == true) {
					iPrevX = iX;
					iPrevY = iY;
					bFirst = false;
				}
				
				graphics.drawLine(iPrevX, iPrevY, iX, iY);
				
				iPrevX = iX;
				iPrevY = iY;
			}
		}

		// Center graph
		iWidth += iFrameWidth;
		iHeight += iFrameWidth;

		Image imgFrame = new BufferedImage(iWidth, iHeight, BufferedImage.TYPE_INT_RGB);
		Graphics graphicsFrame = imgFrame.getGraphics();
		
		graphicsFrame.setColor(Color.WHITE);
		graphicsFrame.fillRect(0, 0, iWidth, iHeight);
		
		graphicsFrame.drawImage(img, (iFrameWidth / 2), (iFrameWidth / 2), null);

		// Display image		
		this.lblPopulationGraph.setIcon(new ImageIcon(imgFrame));
	}
}
