package binpackage;

import exceptions.NoContextException;

/**
 * This is the base class for all problem generators, greedy solution finders and
 * optimizers. It ensuers that all of them can work with a valid context.
 * @author dennis
 */
public abstract class ProblemWorker {

	/**
	 * This string contains the name of the object to be displayed in the UI.
	 */
	public static String strName;
	
	/**
	 * The context is used to represent the problem and all necessary parameters.
	 */
	protected Context cxt = null;

	/**
	 * This constructor initializes the object with a default context.
	 * @param p_cxt The default context.
	 */
	public ProblemWorker(Context p_cxt) {
		this.cxt = p_cxt;
	}

	/**
	 * This method checks if a valid context is present. It throws an exception if no
	 * context is available.
	 * 
	 * @throws NoContextException
	 */
	protected void checkContext() throws NoContextException {
		if (this.cxt == null) {
			throw new NoContextException();
		}
	}

}